//==============================================================================
// AVCR Experiment for Unreal Tournament 2004
// Version 1.2
// 31/10/2008
// Author - Michal Bida, michal.bida@seznam.cz
//==============================================================================
class AVCR_Experiment extends DeathMatch
	config(AVCR);

#exec OBJ LOAD FILE=TeamSymbols_UT2003.utx
#exec OBJ LOAD FILE=TeamSymbols.utx				// needed right now for Link symbols, etc.
#exec OBJ LOAD File=AnnouncerMain.uax
#exec OBJ LOAD File=PsuAVCR.usx

replication
{

	reliable if ( Role==ROLE_Authority )
		GoalBeforeStart, GoalAfterEnd, ExperimentBeeping, ExperimentCount;
}

var string GameClass;
var string ExperimentStartTime;

//This is used for logging used ports for bot and control server
var FileLog Logger;
var Actor FocusActor;

var Actor Goal;
var Actor Start;
var Actor Mark0;
var Actor Mark1;
var Actor Mark2;
var Actor Mark3;
var Actor Mark4;
var Actor Mark5;

//possible goal positions for the experiments
var config vector CircleLocList[64];

var config vector MarkLocList[8];
var config rotator MarkRotList[8];
var array<Actor> MarkList;

//specifies experiments
var config int MarkPositions[64];
var config int StartDotPositions[64];
var config int GoalPositions[64];

var config byte GoalBeforeStart[64];
var config byte GoalAfterEnd[64];
var config byte ExperimentBeeping[64];

var float experiment_delay;
var config float MarkHeight;

/*
var config bool bShowGoalBeforeStart;
var config bool bShowGoalAfterEnd;
var config bool bBeeping;
  */

var bool bExperimentKeyPressed;

var int ExperimentCount;
var config int ExperimentMaxCount;
var config int ChoosedMark;

var config bool bRandomize;



//distance from goal when beeping starts
var config float approachThreshold;

//when the beeping started for the first time
var int playerBeepIndex;

var bool bBeepingIndexSet;

var string beepTime;

//helper vectors for some distance counting
var vector vect1, vect2;

var AVCRxPlayer Experimenter;


//This function is automaticaly called after beginning of the game
simulated function PostBeginPlay()
{

	Super.PostBeginPlay();
	FocusActor = spawn(class'AVCR.AVCR_FocusActor', self);
	FocusActor.SetDrawScale(0.2);
	FocusActor.bHidden = false;

	Goal = Spawn(class'AVCR.AVCR_Circle', self);
	Goal.SetRotation(rot(0, 32768, 0));

	Start = Spawn(class'AVCR.AVCR_Point', self);
	//Start = Spawn(class'BotAPI.FocusActorClass', self);
	//FocusActor.SetDrawScale(0.2);

	MarkList[0] = Spawn(class'AVCR.AVCR_MarkGreen1',self);
	MarkList[1] = Spawn(class'AVCR.AVCR_MarkGreen2',self);
	MarkList[2] = Spawn(class'AVCR.AVCR_MarkGreen3',self);
	MarkList[3] = Spawn(class'AVCR.AVCR_MarkRed1',self);
	MarkList[4] = Spawn(class'AVCR.AVCR_MarkRed2',self);
	MarkList[5] = Spawn(class'AVCR.AVCR_MarkRed3',self);

	InitVariables();

    //Test();
}

event InitGame(string Options, out string Error )
{
	local Mutator M;
	local bool bFound;

	super.InitGame(Options, Error);

	bFound = false;
	//Make sure the AVCR_Mutator is on!
	for (M = BaseMutator; M != None; M = M.NextMutator)
	{
    	if (M.IsA('AVCR_Mutator'))
    	{
    		bFound = true;
			break;
		}

	}
	if (!bFound)
		AddMutator("AVCR.AVCR_Mutator");
}

function InitVariables()
{
	local int i,j,Threshold;

	experiment_delay = 5; //(5 seconds)
	ExperimentCount = 0;

	if (bRandomize)
	{
		//randomize Mark and star pos. here
		MarkPositions[0] = Rand(7);
 		for (i=1;i<64;i++)
		{
			MarkPositions[i] = Rand(7);
			while (MarkPositions[i]==MarkPositions[i-1])
			{
				StartDotPositions[i]=Rand(7);
			}
		}

		for (i=0;i<64;i++)
		{
			StartDotPositions[i] = Rand(7);
			while (StartDotPositions[i]==MarkPositions[i])
			{
				StartDotPositions[i]=Rand(7);
			}
		}

		for (j=0;j<64;j++)
		{
			if (CircleLocList[j] == vect(0,0,0))
			{
				break;
				Threshold = j-1;
			}
		}

		for (i=0;i<64;i++)
		{
			GoalPositions[i] = Rand(Threshold);
		}
	}

	Log("AVCR Experiment Scenario Ready.");
}

simulated function UpdateHeight(float newHeight)
{
	local int i;
	local vector v;

	MarkHeight = newHeight;

	for(i = 0; i < 8;i = i + 1)
	{
		MarkLocList[i].z=MarkHeight;
	}

	v.x = MarkList[ChoosedMark].Location.x;
	v.y = MarkList[ChoosedMark].Location.y;
	v.z = MarkHeight;
	MarkList[ChoosedMark].SetLocation(v);
}

simulated function Test()
{
	local StaticMeshActor A;

	foreach AllActors(class'Engine.StaticMeshActor', A,)
	{
		log("Actor is "$A);
		A.bHidden = true;
		A.bAlwaysRelevant = false;
		log("Acotr bHidden is "$A.bHidden);
	}
}

event PlayerController Login
(
    string Portal,
    string Options,
    out string Error
)
{
    local PlayerController NewPlayer;

    NewPlayer = Super.Login(Portal,Options,Error);
    if (Experimenter == none)
       Experimenter = AVCRxPlayer(NewPlayer);

    return NewPlayer;
}

//Here we store information about our game server into special file
function LogExperiment()
{
	local int i;
	local string FileName;

	Logger = Spawn(class'Engine.FileLog',self);

	FileName = Level.Year$"-"$Level.Month$"-"$Level.Day$"_"$Level.Hour$"-"$Level.Minute$"-"$Level.Second$"-Experiment_"$ExperimentCount;
	if (GoalBeforeStart[ExperimentCount] == 1)
		FileName = FileName $ "_SGBef";
	if (GoalAfterEnd[ExperimentCount] == 1)
		FileName = FileName $ "_SGAft";

	if (ExperimentBeeping[ExperimentCount] == 1)
		FileName = FileName $ "_beep";

	Logger.OpenLog(FileName);

	Logger.Logf("Start: "$ExperimentStartTime);
	Logger.Logf("End: "$Level.Year$"/"$Level.Month$"/"$Level.Day$" "$Level.Hour$":"$Level.Minute$":"$Level.Second$":"$Level.MilliSecond);
	Logger.Logf("ShowGoalBeforeStart: "$GoalBeforeStart[ExperimentCount]);
	Logger.Logf("ShowGoalAfterEnd: "$GoalAfterEnd[ExperimentCount]);
	Logger.Logf("Beeping: "$ExperimentBeeping[ExperimentCount]);

	Logger.Logf("PlayerEndPosition: "$Experimenter.Pawn.Location);
	Logger.Logf("GoalPosition: "$Goal.Location);
	Logger.Logf("MarkPosition: "$MarkList[ChoosedMark].Location);
	Logger.Logf("StartPosition: "$Start.Location);
	Logger.Logf("PlayerSpeed: "$Experimenter.Pawn.GroundSpeed);

	Logger.Logf("//------------------------//");
	Logger.Logf("//------------------------//");
	Logger.Logf("PlayerPath: ");
	for (i = 0;i < Experimenter.PlayerPosList.Length; i=i+1 )
	{
		if (i == playerBeepIndex)
		{
			Logger.Logf("First Beep! Time: " $ beepTime);
		}
		Logger.Logf(string(Experimenter.PlayerPosList[i]));
	}

	Logger.CloseLog();
	Logger.Destroy();

	Broadcast(self,"Log written to UT2004/UserLogs/"$FileName$".log");

	ConsoleCommand("flush", True);

}

simulated function Tick(float DeltaTime)
{
	local Controller C;

	super.Tick(DeltaTime);
	//log("DeltaTime is "$DeltaTime);
	for( C = Level.ControllerList; C != None; C = C.NextController )
	{
		if (C.IsA('AVCRxPlayer'))
		{
			if (C.Pawn != none)
			{
			    FocusActor.SetLocation(C.Pawn.Location);
			}
		}
	}
}

state ExperimentInit
{
Begin:
	//log("ExperimentStart");
	if (Experimenter != none)
		Experimenter.NotifyClient();
	Broadcast(self,"Prepare for new experiment round");
	sleep(1);
	Broadcast(self,"Experiment INITIATING");
	InitExperiment();
	Start.bHidden = false;
	bExperimentKeyPressed = false;
	sleep(1);
	Broadcast(self,"Confirm you have reached red dot by pressing number 1 key");
WaitingForReachingStart1:
	sleep(0.2);
	if (!bExperimentKeyPressed)
		goto 'WaitingForReachingStart1';
	gotoState('ExperimentStart','Begin');
}

state ExperimentStart
{
Begin:
	if (Experimenter != none)
		Experimenter.NotifyClient();
	Start.bHidden = true;
	if (GoalBeforeStart[ExperimentCount] == 1)
		Goal.bHidden = false;
	MarkList[ChoosedMark].bHidden = false;
	Broadcast(self,"Red dot reached, for experiment start press number 2 key");
	bExperimentKeyPressed = false;
WaitingForStart1:
	sleep(0.2);
	if (!bExperimentKeyPressed)
		goto 'WaitingForStart1';
	ExperimentStarted();
	gotoState('ExperimentRunning','Begin');
}

state ExperimentRunning
{

Begin:
	//log("ExperimentRunning");
	if (Experimenter != none)
		Experimenter.NotifyClient();
	Broadcast(self,"Experiment RUNNING");
	sleep(1.0);
	bBeepingIndexSet = false;
	playerBeepIndex = -1;
	beepTime = "";
	if (ExperimentBeeping[ExperimentCount] == 1)
	{
		Broadcast(self,"When you are close to target, you'll hear beep.");
		Broadcast(self,"When you think you've reached the goal, press number 3 key");
		goto 'Beeping';
	}
	else
	{
		Broadcast(self,"When you think you've reached the goal, press number 3 key");
		goto 'Running';
	}
Running:
	sleep(1.0);
	goto 'Running';
Beeping:
	sleep(0.3);
	vect1 = Goal.Location;
	vect2 = Experimenter.Pawn.Location;
	vect1.z = 0;
	vect2.z = 0;
	if (vSize( vect1 - vect2 ) < approachThreshold)
	{
		Experimenter.PlayOwnedSound(Sound'WeaponSounds.LockOn', SLOT_None, 5.0);
		if (!bBeepingIndexSet)
		{
			playerBeepIndex = Experimenter.PlayerPosList.Length;
			beepTime = Level.Year $"/"$ Level.Month $"/"$ Level.Day $" "$ Level.Hour $":"$ Level.Minute$":"$Level.Second$":"$Level.MilliSecond;
			bBeepingIndexSet = true;
		}
		//gotoState('ExperimentEnd','Begin');
		//PlaySound(Sound'CicadaSnds.Decoy.DecoyLaunch', SLOT_None, 5.0);
		//Sound'WeaponSounds.LockOn'
	}
	goto 'Beeping';
}

state ExperimentEnd
{
Begin:
	//log("ExperimentEnd");
	if (Experimenter != none)
		Experimenter.NotifyClient();
	Broadcast(self,"Experiment ENDED");
	ExperimentEnded();
	sleep(1);
End:
	Broadcast(self,"Waiting for key number 4 or HOME key to be pressed for a new round");
	bExperimentKeyPressed = false;
WaitingForKeyPress:
	sleep(0.2);
	if (!bExperimentKeyPressed)
		goto 'WaitingForKeyPress';
	ExperimentCount += 1;
	if (ExperimentCount >= ExperimentMaxCount)
		ExperimentCount = 0;
	gotoState('ExperimentInit','Begin');
}

function InitExperiment()
{
	local int i,k,l,m,a;
	local rotator angle;
	local vector GoalPosition, temp, RealPosDirection, usedvec;

	if (ExperimentCount >= ExperimentMaxCount)
		ExperimentCount = 0;

	i = MarkPositions[ExperimentCount % ExperimentMaxCount];
	l = StartDotPositions[ExperimentCount % ExperimentMaxCount];
	m = GoalPositions[ExperimentCount % ExperimentMaxCount];

    for( k = 0; k < MarkList.Length; k = k + 1 )
	{
		MarkList[k].bHidden = true;
	}

	Goal.bHidden = true;
	Start.bHidden = true;

   	MarkList[ChoosedMark].SetLocation(MarkLocList[i]);
	MarkList[ChoosedMark].SetRotation(MarkRotList[i]);

	Start.SetLocation(MarkLocList[l]);
	Start.SetRotation(MarkRotList[l]);

	Goal.SetLocation(CircleLocList[m]);

	//vec130 = vect(-1,1.19,0);
	//vec15 = vect(1,0.268,0);
	//angle = rot(23673, 0, 0); //130 degrees

	//RealPosDirection += Normal(usedvec) * (RealPosDirection dot Normal(usedvec)) * -1;
    //RealPosDirection = Normal(Normal(RealPosDirection * (1 - (a / 90) ) ) + (Normal(RealPosDirection cross vect(0, 0, 1)) * (a / 90)));

 	log("Goal is "$Goal$" location: "$Goal.Location);
	log("Mark is "$MarkList[ChoosedMark]$" location: "$MarkList[ChoosedMark].Location);

}

function ExperimentStarted()
{
	local Controller C;

    ExperimentStartTime = Level.Year$"/"$Level.Month$"/"$Level.Day$" "$Level.Hour$":"$Level.Minute$":"$Level.Second$":"$Level.MilliSecond;
	Goal.bHidden = true;
	Experimenter.bExperimentOn = true;
	Experimenter.gotoState('PlayerWalking','Running');
	Experimenter.NotifyClient();
	/*
    for( C = Level.ControllerList; C != None; C = C.NextController )
	{
		if (C.IsA('AVCRxPlayer') && !C.PlayerReplicationInfo.bIsSpectator)
		{

			AVCRxPlayer(C).bExperimentOn = true;
			AVCRxPlayer(C).gotoState('PlayerWalking','Running');
			AVCRxPlayer(C).NotifyClient();
			BroadCast(self,"Controller "$C.PlayerReplicationInfo.PlayerName);
		}
	}*/
}

function ExperimentEnded()
{
	local Controller C;

	if (GoalAfterEnd[ExperimentCount] == 1)
		Goal.bHidden = false;
	LogExperiment();
	Experimenter.bExperimentOn = false;
	Experimenter.gotoState('PlayerWalking','Waiting');
	Experimenter.NotifyClient();
	Experimenter.PlayerPosList.Length = 0;
	/*
    for( C = Level.ControllerList; C != None; C = C.NextController )
	{
		if (C.IsA('AVCRxPlayer') && !C.PlayerReplicationInfo.bIsSpectator)
		{
			AVCRxPlayer(C).bExperimentOn = false;
			AVCRxPlayer(C).gotoState('PlayerWalking','Waiting');
			AVCRxPlayer(C).NotifyClient();
			AVCRxPlayer(C).PlayerPosList.Length = 0;
		}
	}*/
   	//ExperimentCount += 1;
   	//Done elsewhere now
}

//Prevents epic bots from automatically joining the game
function bool NeedPlayers()
{
	return false;
}

//Prevents epic bots from automatically joining the game
function bool BecomeSpectator(PlayerController P)
{
	if ( !Super.BecomeSpectator(P) )
		return false;

	return true;
}

defaultproperties
{
	NetWait=2
	CountDown=0
	bPauseable=True
	TimeLimit=40
	PlayerControllerClassName="AVCR.AVCRxPlayer"
	DefaultPlayerClassName="AVCR.AVCRxPawn"
	GameName="AVCR Experiment"
	GameClass="BotDeathMatch"
	Acronym="AVCR"
	MapPrefix="DM"
	RemoteRole=ROLE_AutonomousProxy
	MarkPositions(0)=6
	MarkPositions(1)=0
	MarkPositions(2)=7
	MarkPositions(3)=3
	MarkPositions(4)=4
	MarkPositions(5)=1
	MarkPositions(6)=5
	MarkPositions(7)=2
	StartDotPositions(0)=2
	StartDotPositions(1)=3
	StartDotPositions(2)=5
	StartDotPositions(3)=7
	StartDotPositions(4)=1
	StartDotPositions(5)=6
	StartDotPositions(6)=0
	StartDotPositions(7)=4
	GoalPositions(0)=6
	GoalPositions(1)=0
	GoalPositions(2)=7
	GoalPositions(3)=3
	GoalPositions(4)=4
	GoalPositions(5)=1
	GoalPositions(6)=5
	GoalPositions(7)=2
	CircleLocList(0)=(X=-337.000000,Y=267.000000,Z=-241.000000)
	CircleLocList(1)=(X=-329.000000,Y=207.000000,Z=-241.000000)
	CircleLocList(2)=(X=-281.000000,Y=175.000000,Z=-241.000000)
	CircleLocList(3)=(X=-225.000000,Y=187.000000,Z=-241.000000)
	CircleLocList(4)=(X=-193.000000,Y=231.000000,Z=-241.000000)
	CircleLocList(5)=(X=-201.000000,Y=287.000000,Z=-241.000000)
	CircleLocList(6)=(X=-245.000000,Y=323.000000,Z=-241.000000)
	CircleLocList(7)=(X=-297.000000,Y=319.000000,Z=-241.000000)
	MarkLocList(0)=(X=-364.999756,Y=350.999939,Z=-143)
	MarkLocList(1)=(X=-399.999756,Y=248.000122,Z=-143)
	MarkLocList(2)=(X=-360.000122,Y=143.999863,Z=-143)
	MarkLocList(3)=(X=-265.999908,Y=108.999954,Z=-143)
	MarkLocList(4)=(X=-171,Y=146.999985,Z=-143)
	MarkLocList(5)=(X=-127.000061,Y=248.999939,Z=-143)
	MarkLocList(6)=(X=-161.999878,Y=343.999695,Z=-143)
	MarkLocList(7)=(X=-264,Y=390,Z=-143)
	MarkRotList(0)=(Pitch=0,Yaw=8192,Roll=0)
	MarkRotList(1)=(Pitch=0,Yaw=16384,Roll=0)
	MarkRotList(2)=(Pitch=0,Yaw=24576,Roll=0)
	MarkRotList(3)=(Pitch=0,Yaw=32768,Roll=0)
	MarkRotList(4)=(Pitch=0,Yaw=40960,Roll=0)
	MarkRotList(5)=(Pitch=0,Yaw=49152,Roll=0)
	MarkRotList(6)=(Pitch=0,Yaw=57344,Roll=0)
	MarkRotList(7)=(Pitch=0,Yaw=0,Roll=0)
	MarkHeight=-143
	ExperimentMaxCount=8
	ChoosedMark = 5
	approachThreshold = 20
}
